# This file is part of the Motelab batch scripts
# Copyright (c) 2005 Bret Hull, Kyle Jamieson, all rights reserved.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA

#
# libml: library routines to perform common motelab scripting tasks.
#

use strict;
use Carp;

sub id2prefix ($$) {
  my $dbh = shift;
  my $id = shift;
  
  my $sth = $dbh->prepare("select dbprefix, state from jobschedule
  where id=$id");

  $sth->execute() or croak "libml: $DBI::errstr";
  my ($prefix, $state) = $sth->fetchrow_array;
  #croak "libml: bad state for job $id" if $prefix and $state != 2;
  return $prefix;
}

sub prefix2id ($$) {
  my $dbh = shift;
  my $prefix = shift;
  
  my $sth = $dbh->prepare("select id from jobschedule where
  dbprefix='$prefix'");
  $sth->execute() or croak "libml: $DBI::errstr";
  my ($id) = $sth->fetchrow_array;
  return $id;
}

sub addtoprefixes ($$$) {
  my $dbh = shift;
  my $prefixes = shift;
  my $ids = shift;

  foreach my $idarg (@$ids) {
    foreach my $idstring (split /,/, $idarg) {
      if ($idstring =~ /^(\d+)$/) {
        my $prefix = id2prefix($dbh, $1);
        push @$prefixes, $prefix if $prefix;
      } elsif ($idstring =~ /^(\d+)-(\d+)$/) {
        for (my $id = $1; $id <= $2; $id++) {
          my $prefix = id2prefix($dbh, $id);
          push @$prefixes, $prefix if $prefix;
        }
      } else {
        croak "libml: bad argument to --id: $idstring";
      }
    }
  }
}

sub prefix2jobid ($$) {
  my $dbh = shift;
  my $prefix = shift;
  
  my $sth = $dbh->prepare("select jobid from jobschedule where
  dbprefix='$prefix'");
  $sth->execute() or croak "libml: $DBI::errstr";
  my ($id) = $sth->fetchrow_array;
  return $id;
}

sub prefix2db ($$) {
  my $dbh = shift;
  my $prefix = shift;
  
  my $sth = $dbh->prepare("select dbname from auth, jobschedule, jobs
  where dbprefix='$prefix' and jobs.id=jobschedule.jobid and
  auth.username=jobs.owner");
  $sth->execute() or croak "libml: $DBI::errstr";
  my ($dbname) = $sth->fetchrow_array;
  return $dbname;
}

sub db2motes ($) {
  my $dbh = shift;
  
  my $sth = $dbh->prepare("select moteid from motes where active=1
  order by moteid");
  $sth->execute() or croak "libml: $DBI::errstr";
  return (map $_->[0], @{$sth->fetchall_arrayref});
}

sub getUserID ($$) {
  my ($user,$dbh) = @_;
  my $sth = $dbh->prepare("SELECT auth.id " .
                          "FROM auth WHERE auth.username=\"$user\"");
  $sth->execute();

  return $sth->fetchrow_array();
}

sub getExecDir ($$) {
  my ($user,$dbh) = @_;
  my $sth = $dbh->prepare("SELECT auth.exec_dir " .
                          "FROM auth WHERE auth.username=\"$user\"");
  $sth->execute();

  return $sth->fetchrow_array();
}

sub ml_default_values (\%) {
  my $options = shift;
  
  $options->{'user'} = $ENV{'ML_DB_USER'}
    if not exists $options->{'user'} and exists $ENV{'ML_DB_USER'};
  croak "libml: specify --user or ML_DB_USER" if not exists $options->{'user'};
  
  $options->{'password'} = $ENV{'ML_DB_PASSWORD'}
    if not exists $options->{'password'} and exists $ENV{'ML_DB_USER'};
  croak "libml: specify --password or ML_DB_PASSWORD" 
    if not exists $options->{'password'};

  $options->{'host'} = $ENV{'ML_DB_HOST'}
    if not exists $options->{'host'} and exists $ENV{'ML_DB_HOST'};
  croak "libml: specify --user or ML_DB_HOST" if not exists $options->{'host'};

  $options->{'port'} = $ENV{'ML_DB_PORT'}
    if not exists $options->{'port'} and exists $ENV{'ML_DB_PORT'};
  croak "libml: specify --port or ML_DB_PORT" if not exists $options->{'port'};
}

1;
