# This file is part of the Motelab batch scripts
# Copyright (c) 2005 Bret Hull, Kyle Jamieson, all rights reserved.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA

#
# ml_joblist: lists all jobs ever run on Motelab.
#
# Options:
#   --count=[i]: limit to i jobs
#   --all: show all users' jobs

use strict;
use Getopt::Long;
use DBI;
use Time::Local;
use Date::Parse;

use libml;

#
# Parse command line options
#
my %options=();

GetOptions(\%options, "user=s", "host=s", "port=s", "password=s",
                      "count=i", "all");
ml_default_values(%options);

$options{'count'} = 50 if not defined $options{'count'};

my $dsn = "DBI:mysql:auth:$options{'host'}:$options{'port'}";
my $dbh = DBI->connect($dsn, $options{'user'}, $options{'password'});
die "Unable to connect to database $options{'host'}" unless $dbh;
print STDERR "Connected to $options{'host'}:$options{'port'} as $options{'user'}\n";

my $sth;

my $query = "SELECT jobid, end, dbprefix, owner,
jobschedule.id FROM jobschedule, jobs WHERE ";

$query .= "owner='$options{'user'}' AND " unless $options{'all'};

$query .= "jobs.id=jobschedule.jobid AND state = 2 ORDER BY UNIX_TIMESTAMP(end) DESC ";

$query .= "LIMIT $options{'count'}" if ($options{'count'});

my $sth = $dbh->prepare($query);
$sth->execute();

my $finished = $sth->fetchall_arrayref;
my $num_finished = scalar @$finished;

my ($lastuser, $lastprefix);
my ($prefixstem, $lastprefixstem);

my @now = localtime;
my @fclist = ();

if ($options{'all'}) {
  print "\nEveryone's $num_finished most recently-finished jobs:\n";
} else {
  print "\nYour $num_finished most recently-finished jobs:\n";
}

foreach my $fin (@$finished) {
  my ($user, $end, $jobnum, $prefix) = 
    ($fin->[3], $fin->[1], $fin->[4], $fin->[2]);

  my $fc;
  my @end = strptime($end);

  # splice out fc setting
  if ($prefix =~ /^(\w+?)fc(\d)(\w+?)_\d+$/) {
    $prefixstem = $1.$3;
    $fc = $2;
  } else {
    printf("\n%-10s %d/%d %-5s %-40s", substr($user, 0, 10), 
            $end[4] + 1, $end[3] , $jobnum, substr($prefix, 0, 40));
    next;
  }
  

  #print "prefixstem=$prefixstem, fc=$2\n";

  if ($user eq $lastuser and $prefixstem eq $lastprefixstem) {
    push @fclist, $fc;
  } else {
    my $displayend;

    if (@fclist) {
      print "$_" for @fclist;
      @fclist = ();
    }

    if ($end[3] eq $now[3] and $end[4] eq $now[4] and 
        $end[5] eq $now[5]) {
      # this is a job from today
      my $ampm;
      if ($end[2] > 12) {
        $end[2] -= 12;
        $ampm='pm';
      } elsif ($end[2] == 12) {
        $ampm='pm';
      } elsif ($end[2] == 0) {
        $end[2] = 12;
        $ampm = 'am';
      } else {
        # kludge to eval as number
        $end[2] = $end[2] + 0;
        $ampm='am';
      }
      
      printf("\n%-10s %s:%s%s %-5s %-40s", substr($user, 0, 10), 
              $end[2], $end[1], $ampm, $jobnum, substr($prefixstem, 0, 40));
    } else {
      # job from not today
      printf("\n%-10s %d/%d %-5s %-40s", substr($user, 0, 10), 
              $end[4] + 1, $end[3] , $jobnum, substr($prefixstem, 0, 40));
    }
    print "fc$fc";
  }

  $lastuser = $user;
  $lastprefix = $prefix;
  $lastprefixstem = $prefixstem;
}

if (@fclist) {
  print "$_" for @fclist;
}
print "\n";
