# This file is part of the Motelab batch scripts
# Copyright (c) 2005 Bret Hull, Kyle Jamieson, all rights reserved.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA

# 
# ml_resched: bumps all jobs that start later than a given time by
# some offset of minutes into the future.
#
# Options:
#   --time=[t]: bump all jobs that start after time t
#   --offset=[min]: bump each job by min minutes
#

use strict;
use Getopt::Long;
use DBI;
use Time::Local;
use Date::Parse;

use libml;

#
# Parse command line options
#
my %options=();
GetOptions(\%options, "user=s", "host=s", "password=s", "time=s",
           "offset=i");
ml_default_values(%options);

die "Please specify --time=<reschedule time>" 
  if (not exists $options{'time'});
die "Please specify --offset=<minutes to offset job>"
  if (not exists $options{'offset'});

die "Cannot reschedule in the past"
  if str2time($options{'time'}) <= time;

die "Cannot move jobs backward"
  if $options{'offset'} < 1;

#
# translate options
#
$options{'time'} = str2time($options{'time'});
$options{'offset'} *= 60;

print "$options{'time'}\n";

my $dsn = "DBI:mysql:auth:$options{'host'}:$options{'port'}";
my $dbh = DBI->connect($dsn, $options{'user'}, $options{'password'});
die "Unable to connect to database $options{'host'}" unless $dbh;
print "Connected to $options{'host'}:$options{'port'} as $options{'user'}\n";

my $sth = $dbh->prepare("SELECT jobs.name, jobschedule.id, start, end
FROM jobschedule,jobs WHERE state=0 and jobs.id=jobschedule.jobid AND
UNIX_TIMESTAMP(start) >= $options{'time'} ORDER BY
UNIX_TIMESTAMP(end) DESC");

$sth->execute or die $DBI::errstr;

while (my ($name, $jobid, $start, $end) = $sth->fetchrow_array) {
  print "$name, $jobid, $start, $end\n";
  my $unix_start = str2time($start);
  my $unix_end = str2time($end);

  my $update_sth = $dbh->do("UPDATE jobschedule SET
  start=FROM_UNIXTIME($unix_start + $options{'offset'}),
  end=FROM_UNIXTIME($unix_end + $options{'offset'}) WHERE id=$jobid");
}
