# This file is part of the Motelab batch scripts
# Copyright (c) 2005 Bret Hull, Kyle Jamieson, all rights reserved.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA

# 
# ml_rmjob: removes jobs and their associated data from the testbed.
#

package Cents;

use strict;
use Getopt::Long;
use DBI;

use libml;

my %options=();

GetOptions(\%options, "user=s", "host=s", "port=s",
"prefix=s@","password=s","id=s@", "droponly", "delschedonly",
"dbname=s");

ml_default_values(%options);

my $dsn = "DBI:mysql:auth:$options{'host'}:$options{'port'}";
my $dbh = DBI->connect($dsn, $options{'user'}, $options{'password'});
die "Unable to connect to database $options{'host'}" unless $dbh;
print "Connected to $options{'host'}:$options{'port'} as $options{'user'}\n";

my $sth;
$options{'prefix'} = [] if not defined $options{'prefix'};

my @job_ids;
foreach my $idstring (@{$options{'id'}}) {
  if ($idstring =~ /^(\d+)$/) {
    push @job_ids, $1;
  } elsif ($idstring =~ /^(\d+)-(\d+)$/) {
    for (my $id = $1; $id <= $2; $id++) {
      push @job_ids, $id;
    }
  } elsif ($idstring =~ /^\d+,(\d+)+/) {
    my @job_ids = split /,/, $idstring;
  } else {
    die "Bad argument to --id: $idstring";
  }
}
 
foreach my $id (@job_ids) { 
  # find out the state of the job
  $sth = $dbh->prepare("SELECT state from jobschedule where id=$id");
  $sth->execute() or die;
  my $state = $sth->fetchrow_array();
  if ($sth->rows() != 1) {
    # there should be only one row
    print "cannot remove job $id\n";
    next;
  }
 
  if(!$options{"delschedonly"} && $state == 2) {
   # non-pending jobs have database tables
    if(not exists $options{"dbname"}) {
      $sth = $dbh->prepare("select dbname from auth, jobschedule, jobs ".
                      "where jobschedule.id='$id' and " . 
                      "jobs.id=jobschedule.jobid and auth.username=jobs.owner");
      $sth->execute() or die;
      $options{"dbname"} = $sth->fetchrow_array;
      if($options{'dbname'} eq "") {
        die "unable to determine database name\n";
      }
    }
    print "dbname=$options{'dbname'}\n"; 
    $sth = $dbh->prepare("SHOW TABLES FROM $options{'dbname'} LIKE\"\%\\_$id\\_\%\"");
    $sth->execute() or die;
    while(my $table = $sth->fetchrow_array()) {
      print "Dropping table $options{'dbname'}.$table\n";
      my $sth2 = $dbh->prepare("DROP TABLE IF EXISTS " .
                               "$options{'dbname'}.$table");
      $sth2->execute() or die;
    }
  }

  if(!$options{"droponly"}) {
    if ($state == 1) {
      # we have a running job, mark it for deletion
      print "marking job $id for deletion\n";
      $sth = $dbh->prepare("UPDATE jobschedule SET state=5 WHERE id=$id");
      $sth->execute();
    } else {
      # remove job from schedule
      $sth = $dbh->prepare("DELETE FROM jobschedule WHERE id=$id");
      $sth->execute() or die;
      print "Deleted job $id from schedule\n";
    }
  }
 }
