# This file is part of the Motelab batch scripts
# Copyright (c) 2005 Bret Hull, Kyle Jamieson, all rights reserved.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA

#
# ml_status: display the status of the testbed on the command line.
#

use strict;
use Getopt::Long;
use DBI;
use libml;

#
# Parse command line options
#
my %options=();

GetOptions(\%options, "user=s", "host=s", "port=s", "password=s");
ml_default_values(%options);

my $dsn = "DBI:mysql:auth:$options{'host'}:$options{'port'}";
my $dbh = DBI->connect($dsn, $options{'user'}, $options{'password'});
die "Unable to connect to database $options{'host'}" unless $dbh;
print "Connected to $options{'host'}:$options{'port'} as $options{'user'}\n";

my $sth = $dbh->prepare("
SELECT state, q1.id, owner, name, DATE_FORMAT(start, \"%a %h:%i%p\"), 
  DATE_FORMAT(end, \"%h:%i%p\") FROM 
  (SELECT state, id, jobid, start, end FROM jobschedule 
    WHERE state = 0 OR state = 1) AS q1 
INNER JOIN
  (SELECT id, name, owner FROM jobs) AS q2 
ON q1.jobid = q2.id 
ORDER BY UNIX_TIMESTAMP(start)");

$sth->execute();

my $pending = $sth->fetchall_arrayref;

#my @start = map $_->[1], @$pending;
#my @end = map $_[2], @$pending;
#my @jobid = map $_[0], @$pending;

my $num_jobs = scalar @$pending;

if ($num_jobs > 0) {
  print "There are $num_jobs active or pending jobs:\n\n";
  printf("%1s %-5s %-5s %-40s %-12s %-5s\n", " ", "id", "owner",
         "name", "start", "end");
  printf("%1s %-5s %-5s %-40s %-12s %-5s\n", " ", "--", "-----",
         "----", "-----", "---");

  foreach my $pref (@$pending) {
    printf("%1s %-5s %-5s %-40s %-12s %-5s\n", 
      ($pref->[0] == 1) ? "*" : " ",
      substr($pref->[1], 0, 5), 
      substr($pref->[2], 0, 5), 
      substr($pref->[3], 0, 40), 
      $pref->[4], $pref->[5]);
  }
} else {
  print "There are no pending jobs.\n";
}

$sth = $dbh->prepare("SELECT jobid, DATE_FORMAT(end, \"%a %D %h:%i%p\"),
dbprefix, owner, jobschedule.id FROM jobschedule, jobs WHERE
jobs.id=jobschedule.jobid AND state = 2 AND owner='$options{'user'}' 
ORDER BY end DESC LIMIT 20");

$sth->execute();

my $finished = $sth->fetchall_arrayref;
my $num_finished = scalar @$finished;

print "\nYour $num_finished most recently finished jobs:\n\n";
printf("%-10s %-16s %-5s %-40s\n", "user", "end", "job #", "prefix");
printf("%-10s %-16s %-5s %-40s\n", "----", "---", "-----", "------");

foreach my $fin (@$finished) {
  printf("%-10s %-16s %-5s %-40s\n", substr($fin->[3], 0, 10), 
         $fin->[1], $fin->[4], substr($fin->[2], 0, 40));
}

$sth = $dbh->prepare("SELECT jobid, DATE_FORMAT(end, \"%a %D %h:%i%p\"),
dbprefix, owner, jobschedule.id FROM jobschedule, jobs WHERE
jobs.id=jobschedule.jobid AND state = 2 ORDER BY end DESC LIMIT 5");

$sth->execute();

my $finished = $sth->fetchall_arrayref;
my $num_finished = scalar @$finished;

print "\nThe $num_finished most recently finished jobs:\n\n";
printf("%-10s %-16s %-5s %-40s\n", "user", "end", "job #", "prefix");
printf("%-10s %-16s %-5s %-40s\n", "----", "---", "-----", "------");

foreach my $fin (@$finished) {
  printf("%-10s %-16s %-5s %-40s\n", substr($fin->[3], 0, 10), 
         $fin->[1], $fin->[4], substr($fin->[2], 0, 40));
}
